import { Server } from "socket.io";
import { TSocket } from "../../utilities/Types";
import { writeFile } from "node:fs/promises";
import path from "path";
import fs from "fs/promises";
import { randomUUID } from 'node:crypto';

interface ITileData {
    [key: string]: Buffer;
}

/**
 * Handle game master upload tile event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
    socket.on('gm:tile:upload', async (data: ITileData, callback: (response: boolean) => void) => {
        try {
            if (socket.character?.role !== 'gm') {
                callback(false);
                return;
            }

            const public_folder = path.join(process.cwd(), 'public', 'tiles');

            // Ensure the folder exists
            await fs.mkdir(public_folder, { recursive: true });

            const uploadPromises = Object.entries(data).map(async ([key, tileData]) => {
                const uuid = randomUUID();
                const filename = `${uuid}.png`;
                const finalFilePath = path.join(public_folder, filename);
                await writeFile(finalFilePath, tileData);
            });

            await Promise.all(uploadPromises);

            callback(true);
        } catch (error) {
            console.error('Error uploading tiles:', error);
            callback(false);
        }
    });
}